# Quoins - A TurboGears blogging system.
# Copyright (C) 2008  James E. Blair <corvus@gnu.org>
# Copyright (C) 2008  Monty Taylor <monty@inaugust.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

from turbogears.command.base import CommandWithDB
from turbogears.database import metadata, session, config

class QuoinsCommand(CommandWithDB):
    desc = "Manage Quoins blog"
    need_project = True

    def __init__(self, version):
        self.find_config()

    def run(self):
        if sys.argv[1]=='create':
            self.create()
        if sys.argv[1]=='init':
            self.init()
        if sys.argv[1]=='migrate-wordpress':
            self.migrate_wordpress()

    def create(self):
        from turbogears.database import bind_meta_data, metadata
        from quoins import blog_model
        from quoins.blog_controllers import get_oid_connection
        from openid.store.sqlstore import MySQLStore
        import MySQLdb

        bind_meta_data()
        metadata.create_all()

        conn = get_oid_connection()
        store = MySQLStore(conn)
        try:
            store.createTables()
        except MySQLdb.OperationalError, message:
            errorcode = message[0]
            if errorcode == 1050:
                print 'ok'
            else:
                raise

    def init(self):
        from quoins.blog_model import Blog

        title = raw_input("Blog title: ")
        subtitle = raw_input("Blog subtitle: ")
        comments = raw_input("Allow comments by default? (y/n) ")
        if comments.strip().lower()=='y':
            comments = True
        else:
            comments = False
        b = Blog()
        b.title = title.strip()
        if subtitle:
            b.subtitle = subtitle.strip()
        b.allow_comments = comments
        session.save(b)
        session.flush()

    def migrate_wordpress(self):
        """ Migrate from a wordpress model """

        from turbogears.database import bind_meta_data

        bind_meta_data()

        schema = raw_input("Wordpress schema: ")
        prefix = raw_input("Wordpress prefix: ")

        from quoins.wordpress_model import config_wp
	config_wp(schema, prefix)

        from quoins.wordpress_model import wp_Post
        from quoins.blog_model import Blog, Post
        from turbogears.util import load_class

        user_class = config.get('identity.saprovider.model.user')
        User = load_class(user_class)

        wp_posts=session.query(wp_Post).select()

        blog = Blog.get(1)

        for wp_post in wp_posts:
            if wp_post.post_status=='publish':
                wp_user = wp_post.author[0]
                try:
                    user=User.selectone_by(user_name=wp_user.user_login)
                except:
                    user=User()
                    user.user_name=wp_user.user_login
                    user.email_address=wp_user.user_email
                    user.display_name=wp_user.user_nicename
                    user.url=wp_user.user_url
                    user.created=wp_user.user_registered
                    session.save(user)
                    session.flush()
                try:
                    post=Post.selectone_by(title=wp_post.post_title)
                except:
                    post=Post()
                    post.blog=blog
                    post.author=user
                    post.title=wp_post.post_title

                    post.teaser=wp_post.post_excerpt
                    post.body=wp_post.post_content
                    teaser_loc = post.body.find(post.teaser)
                    if teaser_loc >= 0:
                        post.body=post.body[:teaser_loc]+post.body[teaser_loc+len(post.teaser):]
                    post.body=post.body.replace("\r\n","<br/>")
                    post.body=post.body.replace("\n\n","<br/>")
                    post.created=wp_post.post_date

                for cat in wp_post.categories:
                    print cat.cat_name, cat.category_nicename, cat.category_description
                    post.tag(cat.cat_name.replace(" ","-").replace("/","-and-"))
                session.save(post)
                session.flush()
        session.flush()
