# Quoins - A TurboGears blogging system.
# Copyright (C) 2008  James E. Blair <corvus@gnu.org>
# Copyright (C) 2008  Monty Taylor <monty@inaugust.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from sqlalchemy import *
from sqlalchemy.orm import backref
from turbogears.database import session, metadata
from sqlalchemy.ext.assignmapper import assign_mapper
from turbogears import identity
import turbogears as tg
import sqlalchemy.sql as sql


class wp_Category(object): pass
class wp_Post(object): pass
class wp_Comment(object): pass
class wp_Link(object): pass
class wp_Option(object): pass
class wp_Postmeta(object): pass
class wp_User(object): pass

# Blog schema

wp_schema = None
wp_prefx = None

def config_wp(schema, prefix):
    global wp_schema, wp_prefix
    wp_schema = schema
    wp_prefix = prefix
    if wp_prefix is not None and wp_prefix[-1] != '_':
        wp_prefix = "%s_" % wp_prefix
    else:
        wp_prefix = ""

    wp_categories_table = Table('%scategories' % wp_prefix, metadata,
                                autoload=True, schema=wp_schema)
    wp_posts_table = Table('%sposts' % wp_prefix , metadata,
                           autoload=True, schema=wp_schema)

    wp_comments_table = Table('%scomments' % wp_prefix, metadata,
                              autoload=True, schema=wp_schema)

    wp_links_table = Table('%slinks' % wp_prefix, metadata,
                           autoload=True, schema=wp_schema)

    wp_options_table = Table('%soptions' % wp_prefix, metadata,
                             autoload=True, schema=wp_schema)

    wp_postmeta_table = Table('%spostmeta' % wp_prefix, metadata,
                             autoload=True, schema=wp_schema)

    wp_users_table = Table('%susers' % wp_prefix, metadata,
                             autoload=True, schema=wp_schema)

    wp_link2cat_table = Table('%slink2cat' % wp_prefix, metadata,
                             autoload=True, schema=wp_schema)

    wp_post2cat_table = Table('%spost2cat' % wp_prefix, metadata,
                              Column('post_id', Integer, ForeignKey('%s.ID'%wp_posts_table)),
                              Column('category_id', Integer, ForeignKey('%s.cat_ID'%wp_categories_table)),
                              autoload=True, schema=wp_schema)

    mapper(wp_Category, wp_categories_table)
    mapper(wp_User, wp_users_table, properties={
           'posts': relation(wp_Post, primaryjoin=
                             wp_users_table.c.ID==wp_posts_table.c.post_author,
                             foreign_keys=[wp_posts_table.c.post_author])
           })
    mapper(wp_Post, wp_posts_table, properties={
            'author':relation(
                wp_User, primaryjoin=
                wp_users_table.c.ID==wp_posts_table.c.post_author,
                foreign_keys=[wp_users_table.c.ID]),
            'categories':relation(
                wp_Category,
                secondary=wp_post2cat_table,
                )
    })
    mapper(wp_Comment, wp_comments_table)

